/*
 *   HP8x.h
 * 
 *   Copyright 2004 Olivier De Smet
 */

//#define DEBUG_ONDISK						// for a dump of debug info on file

#define HP85								// uncomment for an HP85 emulator
// #define HP87								// uncomment for an HP87 emulator
// #define HP7470							// not functional

#define VRAM64K								// 64 K of Video Ram and patched graph modes (only in zoom 2)
											// for HP86/87
#include "capricorn.h"

#define SM_RUN			0					// states of cpu emulation thread
#define SM_INVALID		1
#define SM_RETURN		2

#define VIEW_SHORT  FALSE					// view of disassembler output
#define VIEW_LONG   TRUE

#define	ARRAYSIZEOF(a)	(sizeof(a) / sizeof(a[0]))
#define	_KB(a)			((a)*1024)

#if defined HP87
#define HARDWARE		"Gemini"			// emulator hardware
#endif
#if defined HP85
#define HARDWARE		"Capricorn"
#endif


#define T2CYCLES  (dwCapricornCycles)		// CPU cycles in 4474 Hz time frame

// HP8x.c
#if defined DEBUG_ONDISK
extern HANDLE			hDebug;				// sometimes the Visual debugger don't get all messages ....
#endif
extern HANDLE           hHeap;				// private heap to allocate mem from
extern HPALETTE			hPalette;
extern HPALETTE			hOldPalette;
extern HANDLE			hEventShutdn;
// extern LPTSTR			szAppName;		// not uset as no DDE server
// extern LPTSTR			szTopic;		// not used as no DDE server
extern LPTSTR			szTitle;
extern CRITICAL_SECTION	csGDILock;
extern CRITICAL_SECTION	csKeyLock;
extern HINSTANCE		hApp;
extern HWND				hWnd;
extern HWND				hDlgDebug;
extern HWND             hDlgFind;
extern HDC				hWindowDC;
extern BOOL				bAutoSave;
extern BOOL				bAutoSaveOnExit;
extern BOOL				bAlwaysDisplayLog;
extern HANDLE			hThread;
extern VOID				SetWindowTitle(LPTSTR szString);
extern VOID				UpdateWindowStatus(VOID);
extern VOID				ButtonEvent(BOOL state, UINT nId);
extern LRESULT			OnToolDebug(VOID);

// Settings.c
extern VOID ReadSettings(VOID);
extern VOID WriteSettings(VOID);
extern VOID ReadLastDocument(LPTSTR szFileName, DWORD nSize);
extern VOID WriteLastDocument(LPCTSTR szFilename);

// Display.c
extern UINT   nBackgroundX;
extern UINT   nBackgroundY;
extern UINT   nBackgroundW;
extern UINT   nBackgroundH;
extern UINT   nLcdX;
extern UINT   nLcdY;
extern UINT   nLcdxZoom;
extern UINT	  nLcdyZoom;
extern INT	  nLcdCol[4];
extern INT    nLcdRow[4];
extern HDC    hLcdDC;
extern HDC    hMainDC;
extern VOID   SetLcdColor(UINT nId, UINT nRed, UINT nGreen, UINT nBlue);
extern VOID   CreateLcdBitmap(VOID);
extern VOID   DestroyLcdBitmap(VOID);
extern BOOL   CreateMainBitmap(LPCTSTR szFilename);
extern VOID   DestroyMainBitmap(VOID);
extern VOID   UpdateMainDisplay(BOOL force);
extern VOID   WriteToMainDisplay(BYTE a, DWORD d);
extern VOID   UpdateAnnunciators(VOID);
extern VOID   ResizeWindow(VOID);

// Capricorn.c
extern BOOL    bInterrupt;
extern UINT    nState;
extern UINT    nNextState;
extern BOOL    bRealSpeed;
extern BOOL	   bSound;
extern BOOL    bSetDate;
extern BOOL    bKeySlow;
extern DWORD   dwMaxTimers;
extern WORD    wCyclesTimer;
extern CHIPSET Chipset;
extern DWORD   dwCapricornCycles;
extern HANDLE  hEventDebug;
extern BOOL    bDbgAutoStateCtrl;
extern INT     nDbgState;
extern DWORD   dwDbgStopPC;
extern WORD    dwDbgRstkp;
extern WORD    dwDbgRstk;
extern DWORD   *pdwInstrArray;
extern WORD    wInstrSize;
extern WORD    wInstrWp;
extern WORD    wInstrRp;
extern BYTE	   byLastOp;
extern VOID    SuspendDebugger(VOID);
extern VOID    ResumeDebugger(VOID);
extern VOID    SetSpeed(BOOL bAdjust);
extern BOOL    WaitForIdleState(VOID);
extern UINT    SwitchToState(UINT nNewState);
extern UINT    WorkerThread(LPVOID pParam);
extern VOID	   SetHPTime(VOID);

// Fetch.c
extern WORD    __fastcall EvalOpcode(WORD pc);

// Files.c
extern TCHAR   szHP8xDirectory[MAX_PATH];
extern TCHAR   szCurrentDirectory[MAX_PATH];
extern TCHAR   szCurrentKml[MAX_PATH];
extern TCHAR   szCurrentFilename[MAX_PATH];
extern TCHAR   szBufferFilename[MAX_PATH];
extern LPBYTE  pbyRom;
extern LPBYTE  pbyRomPage[256];
extern BYTE    cCurrentRomType;
extern UINT    nCurrentClass;
extern BOOL    bBackup;
extern VOID    SetWindowLocation(HWND hWnd,INT nPosX,INT nPosY);
extern BOOL    PatchRom(LPCTSTR szFilename);
extern BOOL    MapRom(LPCTSTR szRomDirectory);
extern VOID    UnmapRom(VOID);
extern VOID    ResetDocument(VOID);
extern BOOL    NewDocument(VOID);
extern BOOL    OpenDocument(LPCTSTR szFilename);
extern BOOL    SaveDocument(VOID);
extern BOOL    SaveDocumentAs(LPCTSTR szFilename);
extern BOOL    SaveBackup(VOID);
extern BOOL    RestoreBackup(VOID);
extern BOOL    ResetBackup(VOID);
extern BOOL    GetOpenFilename(VOID);
extern BOOL    GetSaveAsFilename(VOID);
extern BOOL    GetLoadLifFilename(VOID);
extern BOOL    GetSaveLifFilename(VOID);
extern BOOL    GetSaveDiskFilename(LPCTSTR lpstrName);
extern BOOL    LoadLif(LPCTSTR szFilename);
extern BOOL    SaveLif(LPCTSTR szFilename);
extern BOOL    LoadDisk(LPCTSTR szFilename);
extern BOOL    SaveDisk(LPCTSTR szFilename);
extern HBITMAP LoadBitmapFile(LPCTSTR szFilename);

// Iop.c
extern VOID	__fastcall IopCmd(IOP *i);
extern VOID IopReload(VOID);
extern VOID	IopStop(VOID);

// Serial.c
extern VOID *serial_ifc[10];
extern BOOL serial_receive(BYTE *b, BYTE *st);
extern BOOL serial_send_d(BYTE d, BYTE st);
extern BOOL serial_send_c(BYTE d);
extern BOOL serial_reset(IOP *i);
extern VOID serial_stop(IOP *i);
extern VOID DoHP82939(VOID);
extern BOOL hp82939_widle(VOID);
extern VOID hp82939_init(VOID);
extern VOID hp82939_stop(VOID);
extern BOOL s_push(BYTE b, BYTE st);
extern WORD s_out_hi;				
extern WORD s_out_lo;

// Hp-ib.c
extern VOID *hpib_ifc[10];
extern BOOL hpib_receive(BYTE *b, BYTE *st);
extern BOOL hpib_send_d(BYTE d, BYTE st);
extern BOOL hpib_send_c(BYTE d);
extern BOOL hpib_reset(IOP *i);
extern VOID hpib_stop(IOP *i);
extern BOOL h_push(BYTE b, BYTE st);
extern WORD h_out_hi;				
extern WORD h_out_lo;

//Hp-9121.c
extern VOID DoHP9121(VOID);
extern BOOL hp9121_eject(BYTE unit);
extern BOOL hp9121_save(BYTE unit, LPCTSTR szFilename);
extern BOOL hp9121_load(BYTE unit, LPCTSTR szFilename);
extern BOOL hp9121_widle(VOID);
extern VOID hp9121_init(VOID);
extern VOID hp9121_stop(VOID);
extern BOOL hp9121_push_c(BYTE addr, BYTE c);
extern BOOL hp9121_push_d(BYTE addr, BYTE d);
extern BOOL hp9121_ppol_e;

//Hp-2225.c
extern VOID DoHP2225(VOID);
extern BOOL hp2225_eject(VOID);
extern BOOL hp2225_save(VOID);
extern BOOL hp2225_widle(VOID);
extern VOID hp2225_init(VOID);
extern VOID hp2225_stop(VOID);
extern BOOL hp2225_push_c(BYTE addr, BYTE c);
extern BOOL hp2225_push_d(BYTE addr, BYTE d);
extern BOOL hp2225_ppol_e;

// not functional
//Hp-7470.c
#if defined HP7470
extern VOID DoHP7470(VOID);
extern BOOL hp7470_eject(VOID);
extern BOOL hp7470_save(VOID);
extern BOOL hp7470_widle(VOID);
extern VOID hp7470_init(VOID);
extern VOID hp7470_stop(VOID);
extern BOOL hp7470_push_c(BYTE addr, BYTE c);
extern BOOL hp7470_push_d(BYTE addr, BYTE d);
extern BOOL hp7470_ppol_e;
extern VOID Trc_window(VOID);
extern VOID Trc_close(VOID);
extern VOID Trc_init(VOID);
extern VOID Trc_closew(VOID);
#endif

// Mops.c
extern VOID CpuReset(VOID);
extern VOID ReadIO(BYTE *a, BYTE b, BYTE s);
extern VOID WriteIO(BYTE *a, BYTE b, BYTE s);

// Keyboard.c
extern VOID KeyboardEventDown(BYTE nId);
extern VOID KeyboardEventUp(BYTE nId);
extern VOID AsciiEventDown(BYTE nAscii);
extern BOOL WriteToKeyboard(LPBYTE lpData, DWORD dwSize);

#if defined HP85
//Prt_85.c
extern HWND	hWndPrt85;				// handle of printer window
extern BYTE Prt85_get_byte(BYTE pos);
extern VOID Prt85_close(VOID);
extern VOID Prt85_init(VOID);
extern VOID Prt85_print(BYTE car);
extern VOID Prt85_limit(BYTE limit);
extern VOID Prt85_mode(BYTE mode);
#endif

// Sound.c
extern DWORD dwSoundTicks;
extern BOOL bInSound12;
extern BOOL bInSoundxx;
extern BOOL bSoundStop;
extern BOOL InitWaves(VOID);
extern VOID PlayWave12(VOID);
extern VOID PlayWavexx(DWORD dwFreq);
extern VOID HaltWaveInit(VOID);
extern VOID HaltWaveDo(VOID);
extern BOOL CloseWaves(VOID);

// Disasm.c
extern VOID init_disassemble(BYTE arp, BYTE drp);
extern WORD disassemble (WORD addr, LPTSTR out, BOOL view);

// Message Boxes
static __inline int InfoMessage(LPCTSTR szMessage)  {return MessageBox(hWnd, szMessage, szTitle, MB_APPLMODAL|MB_OK|MB_ICONINFORMATION|MB_SETFOREGROUND);}
static __inline int AbortMessage(LPCTSTR szMessage) {return MessageBox(hWnd, szMessage, szTitle, MB_APPLMODAL|MB_OK|MB_ICONSTOP|MB_SETFOREGROUND);}
static __inline int YesNoMessage(LPCTSTR szMessage) {return MessageBox(hWnd, szMessage, szTitle, MB_APPLMODAL|MB_YESNO|MB_ICONEXCLAMATION|MB_SETFOREGROUND);}
static __inline int YesNoCancelMessage(LPCTSTR szMessage) {return MessageBox(hWnd, szMessage, szTitle, MB_APPLMODAL|MB_YESNOCANCEL|MB_ICONEXCLAMATION|MB_SETFOREGROUND);}

// Missing Win32 API calls
static __inline LPTSTR DuplicateString(LPCTSTR szString)
{
	UINT   uLength = lstrlen(szString) + 1;
	LPTSTR szDup   = HeapAlloc(hHeap, 0, uLength*sizeof(szDup[0]));
	lstrcpy(szDup,szString);
	return szDup;
}

