/*
 *   Fetch.c
 *
 *   Copyright 2004 Olivier De Smet
 */

//
//   Opcode fetch dispatcher
//

#include "StdAfx.h"
#include "HP8x.h"
#include "opcodes.h"
#include "mops.h"

#define r2(i) (*((WORD *)(Chipset.R+i)))
#define w2(r, d) *((WORD *)(Chipset.R+r)) = d

typedef struct
{
	const VOID  *pLnk;
	const DWORD dwTyp;				// 1: 2 bytes opcodes, ...
} JMPTAB;

// jump tables
static const JMPTAB oF_[] =
{
	oF0,        1,
	oF1,        1,
	oF2,        1,
	oF3,        1,
	oF4,        1,
	oF5,        1,
	oF6,        1,
	oF7,        1,
	oF8,        1,
	oF9,        1,
	oFA,        1,
	oFB,        1,
	oFC,        1,
	oFD,        1,
	oFE,        1,
	oFF,        1
};

static const JMPTAB oE_[] =
{
	oE0,        0,
	oE1,        0,
	oE2,        0,
	oE3,        0,
	oE4,        0,
	oE5,        0,
	oE6,        0,
	oE7,        0,
	oE8,        0,
	oE9,        0,
	oEA,        0,
	oEB,        0,
	oEC,        0,
	oED,        0,
	oEE,        0,
	oEF,        0
};

static const JMPTAB oD_[] =
{
	oD0,        2,
	oD1,        2,
	oD2,        2,
	oD3,        2,
	oD4,        2,
	oD5,        2,
	oD6,        0,
	oD7,        2,
	oD8,        0,
	oD9,        0,
	oDA,        0,
	oDB,        0,
	oDC,        0,
	oDD,        0,
	oDE,        0,
	oDF,        0
};

static const JMPTAB oC_[] =
{
	oC0,        0,
	oC1,        0,
	oC2,        0,
	oC3,        0,
	oC4,        0,
	oC5,        0,
	oC6,        2,
	oC7,        0,
	oC8,        1,
	oC9,        3,
	oCA,        1,
	oCB,        3,
	oCC,        1,
	oCD,        3,
	oCE,        2,
	oCF,        3
};

static const JMPTAB oB_[] =
{
	oB0,	    2,
	oB1,		2,
	oB2,        2,
	oB3,        2,
	oB4,        2,
	oB5,        2,
	oB6,        2,
	oB7,        2,
	oB8,        2,
	oB9,        2,
	oBA,        2,
	oBB,        2,
	oBC,        2,
	oBD,        2,
	oBE,        2,
	oBF,        2
};

static const JMPTAB oA_[] =
{
	oA0,        0,
	oA1,        0,
	oA2,        0,
	oA3,        0,
	oA4,        0,
	oA5,        0,
	oA6,        0,
	oA7,        0,
	oA8,        1,
	oA9,        3,
	oAA,        1,
	oAB,        3,
	oAC,        0,
	oAD,        0,
	oAE,        0,
	oAF,        0
}; 

static const JMPTAB o9_[] =
{
	o90,        0,
	o91,        0,
	o92,        0,
	o93,        0,
	o94,        0,
	o95,        0,
	o96,        0,
	o97,        0,
	o98,        0,
	o99,        0,
	o9A,        0,
	o9B,        0,
	o9C,        0,
	o9D,        0,
	o9E,        0,
	o9F,        0
};

static const JMPTAB o8_[] =
{
	o80,        0,
	o81,        0,
	o82,        0,
	o83,        0,
	o84,        0,
	o85,        0,
	o86,        0,
	o87,        0,
	o88,        0,
	o89,        0,
	o8A,        0,
	o8B,        0,
	o8C,        0,
	o8D,        0,
	o8E,        0,
	o8F,        0
};

static const JMPTAB o_[] =
{
	oarp,       0,
	oarp,       0,
	oarp,       0,
	oarp,       0,
	odrp,       0,
	odrp,       0,
	odrp,       0,
	odrp,       0,
	o8_,        0,
	o9_,        0,
	oA_,        0,
	oB_,        0,
	oC_,        0,
	oD_,        0,
	oE_,        0,
	oF_,        0
};

//################
//#
//#    Public functions
//#
//################

//
// opcode dispatcher
//
__inline WORD __fastcall EvalOpcode(WORD pc)
{
	JMPTAB const *pJmpTab = o_;

	byLastOp = Nread_nio1(pc);						// read opcode

	pJmpTab = pJmpTab[byLastOp >> 4].pLnk;			// next pointer to table/function
	if (byLastOp >= 0x80)							// another table level ?
		pJmpTab = pJmpTab[byLastOp & 0x0F].pLnk;		// next pointer to table/function
	((VOID (__fastcall *) (WORD)) pJmpTab)((WORD)(pc + 1));		// call function
	pc = r2(4);				// last pc increment done NOW on R4-5
	pc++;											// think about LDM R4,=0xABCD
	w2(4, pc);										// which jump in fact at 0xABCE !
	return pc;
}
