/*
 *   Opcodes.c
 *
 *   Copyright 2004 Olivier De Smet
 */

//
//   Real code for opcodes :)
//

#include "StdAfx.h"
#include "HP8x.h"
#include "opcodes.h"
#include "io.h"								// I/O register definitions
#include "mops.h"

#define w Chipset

#include "Ops.h"

static WORD F_l[64] = {2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,
		  			   2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,
					   8,7,6,5,4,3,2,1,8,7,6,5,4,3,2,1,
				       8,7,6,5,4,3,2,1,8,7,6,5,4,3,2,1};

static WORD F_r[64] = {1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,
		  			   1,2,1,2,1,2,1,2,1,2,1,2,1,2,1,2,
					   1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8,
					   1,2,3,4,5,6,7,8,1,2,3,4,5,6,7,8};

#define fl F_l[w.DRP]
#define fr F_r[w.DRP]

#define r2(i) (*((WORD *)(w.R+i)))

#define r2w(i) ((WORD)(w.R[i] | (w.R[(i+1) & 0x3F]<<8)))

#define w2(r, d) *((WORD *)(w.R+r)) = d

static __forceinline VOID w2w(BYTE r, WORD d)	// write an address in registers with wrap
{
//	w.R[r] = (BYTE) (d & 0x00FF);
	w.R[r] = (BYTE) (d);			// better than previous line ?
	w.R[(++r) & 0x3F] = (BYTE) (d >> 8);
}

#define pcp(d) *((WORD *)(w.R+4)) += d

#define pcm(d) *((WORD *)(w.R+4)) -= d

VOID __fastcall oarp(WORD pc) // ARP
{
	w.dcycles += 2;
	if (byLastOp != 0x01)
		w.ARP = byLastOp;
	else
	{
		w.ARP = w.R[0] & 0x3F;
		w.dcycles += 1;
	}
}

VOID __fastcall odrp(WORD pc) // DRP
{
	w.dcycles += 2;
	if (byLastOp != 0x41)
		w.DRP = byLastOp & 0x3F;
	else
	{
		w.DRP = w.R[0] & 0x3F;
		w.dcycles += 1;
	}
}

VOID __fastcall o80(WORD pc) { w.dcycles += 5; Nesl1(w.R+w.DRP); }										// ELB D
VOID __fastcall o81(WORD pc) { w.dcycles += 4+fl; Nesl(w.R+w.DRP, fl); }								// ELM D
VOID __fastcall o82(WORD pc) { w.dcycles += 5; Nesr1(w.R+w.DRP); }										// ERB D
VOID __fastcall o83(WORD pc) { w.dcycles += 4+fl; Nesr(w.R+w.DRP, fr); }								// ERM D
VOID __fastcall o84(WORD pc) { w.dcycles += 5; Nsl1(w.R+w.DRP); }										// LLB D
VOID __fastcall o85(WORD pc) { w.dcycles += 4+fl; Nsl(w.R+w.DRP, fl); }								// LLM D
VOID __fastcall o86(WORD pc) { w.dcycles += 5; Nsr1(w.R+w.DRP); }										// LRB D
VOID __fastcall o87(WORD pc) { w.dcycles += 4+fl; Nsr(w.R+w.DRP, fr); }								// LRM D
VOID __fastcall o88(WORD pc) { w.dcycles += 5; Ninc1(w.R+w.DRP); }										// ICB D
VOID __fastcall o89(WORD pc) { w.dcycles += 4+fl; Ninc(w.R+w.DRP, fl); }								// ICM D
VOID __fastcall o8A(WORD pc) { w.dcycles += 5; Ndec1(w.R+w.DRP); }										// DCB D
VOID __fastcall o8B(WORD pc) { w.dcycles += 4+fl; Ndec(w.R+w.DRP, fl); }								// DCM D
VOID __fastcall o8C(WORD pc) { w.dcycles += 5; Nneg1(w.R+w.DRP); }										// TCB D
VOID __fastcall o8D(WORD pc) { w.dcycles += 4+fl; Nneg(w.R+w.DRP, fl); }								// TCM D
VOID __fastcall o8E(WORD pc) { w.dcycles += 5; Nnot1(w.R+w.DRP); }										// NCB D
VOID __fastcall o8F(WORD pc) { w.dcycles += 4+fl; Nnot(w.R+w.DRP, fl); }								// NCM D

VOID __fastcall o90(WORD pc) { w.dcycles += 5; Ntst1(w.R[w.DRP]); }									// TSB D
VOID __fastcall o91(WORD pc) { w.dcycles += 4+fl; Ntst(w.R+w.DRP, fl); }								// TSM D
VOID __fastcall o92(WORD pc)																			// CLB D
{ 
	w.dcycles += 5;
	w.R[w.DRP] = 0x00;
	w.Z=TRUE;
	w.CY=FALSE; w.OVF=FALSE;
	w.LSB=FALSE; w.MSB=FALSE;
	w.LDZ=TRUE; w.RDZ=TRUE;
}	
VOID __fastcall o93(WORD pc)																			// CLM D
{
	WORD s = fl;
	BYTE *a = w.R + w.DRP;

	w.dcycles += 4 + s;
	do
	{
		*a++ = 0x00;
	}
	while (--s);
	w.Z=TRUE;
	w.CY=FALSE; w.OVF=FALSE;
	w.LSB=FALSE; w.MSB=FALSE;
	w.LDZ=TRUE; w.RDZ=TRUE;
}		
VOID __fastcall o94(WORD pc) { w.dcycles += 5; NRor1(w.R+w.DRP, w.ARP); }								// ORB D,A
VOID __fastcall o95(WORD pc) { w.dcycles += 4+fl; NRor(w.R+w.DRP, w.ARP, fl); }						// ORM D,A
VOID __fastcall o96(WORD pc) { w.dcycles += 5; NRxr1(w.R+w.DRP, w.ARP); }								// XRB D,A
VOID __fastcall o97(WORD pc) { w.dcycles += 4+fl; NRxr(w.R+w.DRP, w.ARP, fl); }						// XRM D,A
VOID __fastcall o98(WORD pc) { w.dcycles += 4; w.DCM = FALSE; }										// BIN
VOID __fastcall o99(WORD pc) { w.dcycles += 4; w.DCM = TRUE; }											// DCM
VOID __fastcall o9A(WORD pc)																			// SAD
{ 
	BYTE I[4];

	w.dcycles += 8;
	I[0] = (w.OVF ? 0x80:0) | (w.CY ? 0x40:0) | (w.ARP & 0x3F);
	I[1] = (w.OVF ? 0x80:0) | (w.DCM ? 0x40:0) | (w.DRP & 0x3F);
	I[2] = (w.MSB ? 0x80:0) | (w.LDZ ? 0:0x40) | (w.Z ? 0:0x04) | (w.RDZ ? 0:0x02) | w.LSB;
	Nwrite(I, r2(6), 3);
	w2(6, (WORD)(r2(6) + 3));
} 
VOID __fastcall o9B(WORD pc) { w.dcycles += 2; w.E = (w.E-1) & 0x0F; }									// DCE 
VOID __fastcall o9C(WORD pc) { w.dcycles += 2; w.E = (w.E+1) & 0x0F; }									// ICE 
VOID __fastcall o9D(WORD pc) { w.dcycles += 2; w.E = 0; }												// CLE 
VOID __fastcall o9E(WORD pc)																			// RTN
{ 
	w.dcycles += 5;
	w2(6, (WORD)(r2(6) - 2));		// dec SP
	Nread(w.R+4, r2(6), 2);
	pcm(1);
} 
VOID __fastcall o9F(WORD pc)																			// PAD
{
	BYTE I[4];

	w.dcycles += 8;
	w2(6, (WORD)(r2(6) - 3));
	Nread(I, r2(6), 3);
	w.OVF = (I[0] & 0x80);
	w.CY  = (I[0] & 0x40);
	w.ARP = I[0] & 0x3F;
	w.DCM = (I[1] & 0x40);
	w.DRP = I[1] & 0x3F;
	w.MSB = (I[2] & 0x80);
	w.LDZ = !(I[2] & 0x40);
	w.Z   = !(I[2] & 0x04);
	w.RDZ = !(I[2] & 0x02);
	w.LSB = (I[2] & 0x01);
} 

VOID __fastcall oA0(WORD pc) { w.dcycles += 5; NFRcopy1(w.DRP, w.R[w.ARP]); }							// LDB D,A
VOID __fastcall oA1(WORD pc) { w.dcycles += 4+fl; NFRcopy(w.R+w.DRP, w.ARP, fl); }						// LDM D,A
VOID __fastcall oA2(WORD pc) { w.dcycles += 5; NFRstore1(w.ARP, w.R[w.DRP]); }							// STB D,A
VOID __fastcall oA3(WORD pc) { w.dcycles += 4+fl; NFRstore(w.ARP, w.R+w.DRP, fl); }					// STM D,A
VOID __fastcall oA4(WORD pc) { w.dcycles += 6; w.R[w.DRP] = NFread1(r2w(w.ARP)); }						// LDBD D,A
VOID __fastcall oA5(WORD pc) { w.dcycles += 5+fl; NFread(w.R+w.DRP, r2w(w.ARP), fl); }					// LDMD D,A
VOID __fastcall oA6(WORD pc) { w.dcycles += 6; NFwrite1(w.R[w.DRP], r2w(w.ARP)); }						// STBD D,A
VOID __fastcall oA7(WORD pc) { w.dcycles += 5+fl; NFwrite(w.R+w.DRP, r2w(w.ARP), fl); }				// STMD D,A
VOID __fastcall oA8(WORD pc) { w.dcycles += 5; pcp(1); NFcopy1(w.R+w.DRP, Nread_nio1(pc)); }			// LDB D,=o
VOID __fastcall oA9(WORD pc)																			// LDM D,=a
{ 
	w.dcycles += 4+fl; 
	pcp(fl); 
	Nread_nio(w.R+w.DRP, pc, fl);
	Ntst(w.R+w.DRP, fl); 
}
VOID __fastcall oAA(WORD pc) { w.dcycles += 5; pcp(1); NFwrite1(w.R[w.DRP], pc); }						// STB D,=o
VOID __fastcall oAB(WORD pc) { w.dcycles += 4+fl; NFwrite(w.R+w.DRP, pc, fl); pcp(fl); }				// STM D,=a
VOID __fastcall oAC(WORD pc)																			// LDBI D,A	
{ 
	WORD ea;

	w.dcycles += 8;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	w.R[w.DRP] = NFread1(ea); 
}
VOID __fastcall oAD(WORD pc)																			// LDMI D,A	
{ 
	WORD ea;

	w.dcycles += 7+fl;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFread(w.R+w.DRP, ea, fl); 
}
VOID __fastcall oAE(WORD pc)																			// STBI D,A	
{ 
	WORD ea;

	w.dcycles += 8;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFwrite1(w.R[w.DRP], ea); 
}
VOID __fastcall oAF(WORD pc)																			// STMI D,A	
{ 
	WORD ea;

	w.dcycles += 7+fl;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFwrite(w.R+w.DRP, ea, fl); 
}

VOID __fastcall oB0(WORD pc)																			// LDBD D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 6;
	pcp(2); 
	w.R[w.DRP] = NFread1(ea); 
}
VOID __fastcall oB1(WORD pc)																			// LDMD D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 5+fl; 
	pcp(2); 
	NFread(w.R+w.DRP, ea, fl); 
}
VOID __fastcall oB2(WORD pc)																			// STBD D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 6; 
	pcp(2); 
	NFwrite1(w.R[w.DRP], ea); 
}
VOID __fastcall oB3(WORD pc)																			// STMD D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 5+fl; 
	pcp(2); 
	NFwrite(w.R+w.DRP, ea, fl); 
}
VOID __fastcall oB4(WORD pc)																			// LDBD D,X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 8;
	pcp(2); 
	w2(2, (WORD)(ea + r2w(w.ARP)));
	w.R[w.DRP] = NFread1(r2(2)); 
}
VOID __fastcall oB5(WORD pc)																			// LDMD D,X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 7+fl; 
	pcp(2);
	w2(2, (WORD)(ea + r2w(w.ARP)));
	NFread(w.R+w.DRP, r2(2), fl); 
}
VOID __fastcall oB6(WORD pc)																			// STBD D,X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 8; 
	pcp(2);
	w2(2, (WORD)(ea + r2w(w.ARP)));
	NFwrite1(w.R[w.DRP], r2(2));
}
VOID __fastcall oB7(WORD pc)																			// STMD D,X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 7+fl; 
	pcp(2);
	w2(2, (WORD)(ea + r2w(w.ARP)));
	NFwrite(w.R+w.DRP, r2(2), fl); 
}
VOID __fastcall oB8(WORD pc)																			// LDBI D,=a	
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 8;
	pcp(2);
	Nread((BYTE *)(&ea), ea, 2);
	w.R[w.DRP] = NFread1(ea); 
}
VOID __fastcall oB9(WORD pc)																			// LDMI D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 7+fl;
	pcp(2);
	Nread((BYTE *)(&ea), ea, 2);
	NFread(w.R+w.DRP, ea, fl); 
}
VOID __fastcall oBA(WORD pc)																			// STBI D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 8;
	pcp(2);
	Nread((BYTE *)(&ea), ea, 2);
	NFwrite1(w.R[w.DRP], ea); 
}
VOID __fastcall oBB(WORD pc)																			// STMI D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 7+fl;
	pcp(2);
	Nread((BYTE *)(&ea), ea, 2);
	NFwrite(w.R+w.DRP, ea, fl); 
}
VOID __fastcall oBC(WORD pc)																			// LDBI D,X,a	
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 10;
	pcp(2);
	w2(2, (WORD)(ea + r2w(w.ARP)));
	Nread((BYTE *)(&ea), r2(2), 2);
	w.R[w.DRP] = NFread1(ea); 
}
VOID __fastcall oBD(WORD pc)																			// LDMI D,X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 9+fl;
	pcp(2);
	w2(2, (WORD)(ea + r2w(w.ARP)));
	Nread((BYTE *)(&ea), r2(2), 2);
	NFread(w.R+w.DRP, ea, fl); 
}
VOID __fastcall oBE(WORD pc)																			// STBI D,X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 10;
	pcp(2);
	w2(2, (WORD)(ea + r2w(w.ARP)));
	Nread((BYTE *)(&ea), r2(2), 2);
	NFwrite1(w.R[w.DRP], ea); 
}
VOID __fastcall oBF(WORD pc)																			// STMI D,X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 9+fl;
	pcp(2);
	w2(2, (WORD)(ea + r2w(w.ARP)));
	Nread((BYTE *)(&ea), r2(2), 2);
	NFwrite(w.R+w.DRP, ea, fl); 
}

VOID __fastcall oC0(WORD pc) { w.dcycles += 5; Ncm1(w.R+w.DRP, w.R[w.ARP]); }							// CMB D,A
VOID __fastcall oC1(WORD pc) { w.dcycles += 4+fl; NRcm(w.R+w.DRP, w.ARP, fl); }						// CMM D,A
VOID __fastcall oC2(WORD pc) { w.dcycles += 5; Nadd1(w.R+w.DRP, w.R[w.ARP]); }							// ADB D,A
VOID __fastcall oC3(WORD pc) { w.dcycles += 4+fl; NRadd(w.R+w.DRP, w.ARP, fl); }						// ADM D,A
VOID __fastcall oC4(WORD pc) { w.dcycles += 5; Nsub1(w.R+w.DRP, w.R[w.ARP]); }							// SBB D,A
VOID __fastcall oC5(WORD pc) { w.dcycles += 4+fl; NRsub(w.R+w.DRP, w.ARP, fl); }						// SBM D,A
VOID __fastcall oC6(WORD pc)																			// JSB X,a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 11; 
	pcp(3);
	Nwrite(w.R+4, r2(6), 2);
	w2(6, (WORD)(r2(6) + 2));
	w2(2, (WORD)(ea + r2w(w.ARP))); 
	w2(4, (WORD)(r2(2) - 1)); 
}
VOID __fastcall oC7(WORD pc) { w.dcycles += 4+fl; NRand(w.R+w.DRP, w.ARP, fl); }						// ANM D,A
VOID __fastcall oC8(WORD pc) { w.dcycles += 5; pcp(1); Ncm1(w.R+w.DRP, Nread_nio1(pc)); }				// CMB D,=o
VOID __fastcall oC9(WORD pc)																			// CMM D,=a
{ 
	BYTE I[8];

	w.dcycles += 4+fl; 
	Nread_nio(I, pc, fl); 
	pcp(fl); 
	Ncm(w.R+w.DRP, I, fl); 
}
VOID __fastcall oCA(WORD pc) { w.dcycles += 5; pcp(1); Nadd1(w.R+w.DRP, Nread_nio1(pc)); }				// ADB D,=o
VOID __fastcall oCB(WORD pc)																			// ADM D,=a
{ 
	BYTE I[8];

	w.dcycles += 4+fl; 
	Nread_nio(I, pc, fl); 
	pcp(fl); 
	Nadd(w.R+w.DRP, I, fl); 
}
VOID __fastcall oCC(WORD pc) { w.dcycles += 5; pcp(1); Nsub1(w.R+w.DRP, Nread_nio1(pc)); }				// SBB D,=o
VOID __fastcall oCD(WORD pc)																			// SBM D,=a
{ 
	BYTE I[8];

	w.dcycles += 4+fl; 
	Nread_nio(I, pc, fl);
	pcp(fl); 
	Nsub(w.R+w.DRP, I, fl); 
}
VOID __fastcall oCE(WORD pc)																			// JSB =a
{
	WORD ea = Nread_nio2(pc);

	w.dcycles += 9; 
	pcp(3);
	Nwrite(w.R+4, r2(6), 2);
	w2(6, (WORD)(r2(6) + 2));
	w2(4, (WORD)(ea - 1));
}
VOID __fastcall oCF(WORD pc)																			// ANM D,=a
{ 
	BYTE I[8];

	w.dcycles += 4+fl; 
	Nread_nio(I, pc, fl);
	pcp(fl); 
	Nand(w.R+w.DRP, I, fl); 
}

VOID __fastcall oD0(WORD pc)																			// CMBD D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 6; 
	pcp(2); 
	Ncm1(w.R+w.DRP, Nread1(ea)); 
}
VOID __fastcall oD1(WORD pc)																			// CMMD D,=a
{ 
	BYTE I[8];
	WORD ea = Nread_nio2(pc);

	w.dcycles += 5+fl; 
	pcp(2); 
	Nread(I, ea, fl); 
	Ncm(w.R+w.DRP, I, fl); 
}
VOID __fastcall oD2(WORD pc)																			// ADBD D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 6; 
	pcp(2); 
	Nadd1(w.R+w.DRP, Nread1(ea)); 
}
VOID __fastcall oD3(WORD pc)																			// ADMD D,=a
{ 
	BYTE I[8];
	WORD ea = Nread_nio2(pc);

	w.dcycles += 5+fl; 
	pcp(2); 
	Nread(I, ea, fl); 
	Nadd(w.R+w.DRP, I, fl); 
}
VOID __fastcall oD4(WORD pc)																			// SBBD D,=a
{ 
	WORD ea = Nread_nio2(pc);

	w.dcycles += 6; 
	pcp(2); 
	Nsub1(w.R+w.DRP, Nread1(ea)); 
}
VOID __fastcall oD5(WORD pc)																			// SBMD D,=a
{ 
	BYTE I[8];
	WORD ea = Nread_nio2(pc);

	w.dcycles += 5+fl; 
	pcp(2); 
	Nread(I, ea, fl); 
	Nsub(w.R+w.DRP, I, fl); 
}
VOID __fastcall oD6(WORD pc) { w.dcycles += 2; }														// ???
VOID __fastcall oD7(WORD pc)																			// ANMD D,=a
{ 
	BYTE I[8];
	WORD ea = Nread_nio2(pc);

	w.dcycles += 5+fl; 
	pcp(2); 
	Nread(I, ea, fl); 
	Nand(w.R+w.DRP, I, fl); 
}
VOID __fastcall oD8(WORD pc) { w.dcycles += 6; Ncm1(w.R+w.DRP, Nread1(r2w(w.ARP))); }					// CMBD D,A
VOID __fastcall oD9(WORD pc)																			// CMMD D,A
{ 
	BYTE I[8];

	w.dcycles += 5+fl; 
	Nread(I, r2w(w.ARP), fl); 
	Ncm(w.R+w.DRP, I, fl); 
}
VOID __fastcall oDA(WORD pc) { w.dcycles += 6; Nadd1(w.R+w.DRP, Nread1(r2w(w.ARP))); }					// ADBD D,A
VOID __fastcall oDB(WORD pc)																			// ADMD D,A
{ 
	BYTE I[8];

	w.dcycles += 5+fl; 
	Nread(I, r2w(w.ARP), fl); 
	Nadd(w.R+w.DRP, I, fl); 
}
VOID __fastcall oDC(WORD pc) { w.dcycles += 6; Nsub1(w.R+w.DRP, Nread1(r2w(w.ARP))); }					// SBBD D,A
VOID __fastcall oDD(WORD pc)																			// SBMD D,A
{ 
	BYTE I[8];

	w.dcycles += 5+fl; 
	Nread(I, r2w(w.ARP), fl); 
	Nsub(w.R+w.DRP, I, fl); 
}
VOID __fastcall oDE(WORD pc) { w.dcycles += 2; }														// ???
VOID __fastcall oDF(WORD pc)																			// ANMD D,A
{ 
	BYTE I[8];

	w.dcycles += 5+fl; 
	Nread(I, r2w(w.ARP), fl); 
	Nand(w.R+w.DRP, I, fl); 
}

VOID __fastcall oE0(WORD pc)																			// POBD D,+A
{
	w.dcycles += 6;
	w.R[w.DRP] = NFread1(r2w(w.ARP));
	w2w(w.ARP, (WORD)(r2w(w.ARP) + 1));
}
VOID __fastcall oE1(WORD pc)																			// POMD D,+A
{ 
	w.dcycles += 5+fl; 
	NFread(w.R+w.DRP, r2w(w.ARP), fl);
	w2w(w.ARP, (WORD)(r2w(w.ARP) + fl));
}
VOID __fastcall oE2(WORD pc)																			// POBD D,-A
{
	w.dcycles += 6;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - 1));
	w.R[w.DRP] = NFread1(r2w(w.ARP));
}
VOID __fastcall oE3(WORD pc)																			// POMD	D,-A
{
	w.dcycles += 5+fl;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - fl));
	NFread(w.R+w.DRP, r2w(w.ARP), fl);
}
VOID __fastcall oE4(WORD pc)																			// PUBD D,+A
{
	w.dcycles += 6;
	NFwrite1(w.R[w.DRP], r2w(w.ARP));
	w2w(w.ARP, (WORD)(r2w(w.ARP) + 1));
}
VOID __fastcall oE5(WORD pc)																			// PUMD D,+A
{ 
	w.dcycles += 5+fl;
	NFwrite(w.R+w.DRP, r2w(w.ARP), fl);
	w2w(w.ARP, (WORD)(r2w(w.ARP) + fl));
}
VOID __fastcall oE6(WORD pc)																			// PUBD D,-A
{
	w.dcycles += 6;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - 1));
	NFwrite1(w.R[w.DRP], r2w(w.ARP));
}
VOID __fastcall oE7(WORD pc)																			// PUMD D,-A
{
	w.dcycles += 5+fl;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - fl));
	NFwrite(w.R+w.DRP, r2w(w.ARP), fl);
}
VOID __fastcall oE8(WORD pc)																			// POBI D,+A
{ 
	WORD ea;

	w.dcycles += 8;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	w.R[w.DRP] = NFread1(ea);
	w2w(w.ARP, (WORD)(r2w(w.ARP) + 2));	
}
VOID __fastcall oE9(WORD pc)																			// POMI D,+A
{
	WORD ea;

	w.dcycles += 7+fl;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFread(w.R+w.DRP, ea, fl);
	w2w(w.ARP, (WORD)(r2w(w.ARP) + 2));	
}
VOID __fastcall oEA(WORD pc)																			// POBI D,-A
{
	WORD ea;

	w.dcycles += 8;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - 2));	
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	w.R[w.DRP] = NFread1(ea);
}
VOID __fastcall oEB(WORD pc)																			// POMI D,-A
{
	WORD ea;

	w.dcycles += 7+fl;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - 2));	
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFread(w.R+w.DRP, ea, fl);
}
VOID __fastcall oEC(WORD pc)																			// PUBI D,+A
{
	WORD ea;

	w.dcycles += 8;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFwrite1(w.R[w.DRP], ea);
	w2w(w.ARP, (WORD)(r2w(w.ARP) + 2));	
}
VOID __fastcall oED(WORD pc)																			// PUMI D,+A
{
	WORD ea;

	w.dcycles += 7+fl;
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFwrite(w.R+w.DRP, ea, fl);
	w2w(w.ARP, (WORD)(r2w(w.ARP) + 2));	
}
VOID __fastcall oEE(WORD pc)																			// PUBI D,-A
{
	WORD ea;

	w.dcycles += 8;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - 2));	
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFwrite1(w.R[w.DRP], ea);
}
VOID __fastcall oEF(WORD pc)																			// PUMI D,-A
{
	WORD ea;

	w.dcycles += 7+fl;
	w2w(w.ARP, (WORD)(r2w(w.ARP) - 2));	
	Nread((BYTE *)(&ea), r2w(w.ARP), 2);
	NFwrite(w.R+w.DRP, ea, fl);
}

VOID __fastcall oF0(WORD pc)																			// JMP
{ 
	BYTE d;

	w.dcycles += 5; pcp(1);
	d = Nread_nio1(pc);
	w.dcycles += 1; 	
	if (d > 0x7F)
		pcp(d - 0x100); 
	else
		pcp(d);
}
VOID __fastcall oF1(WORD pc)																			//JNO
{
	w.dcycles += 4; pcp(1); 
	if (! Chipset.OVF)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF2(WORD pc)																			//JOD
{
	w.dcycles += 4; pcp(1); 
	if (Chipset.LSB)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF3(WORD pc)																			//JEV
{
	w.dcycles += 4; pcp(1); 
	if (! Chipset.LSB)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF4(WORD pc)																			//JNG
{
	w.dcycles += 4; pcp(1); 
	if ((Chipset.OVF != 0x00) ^ (Chipset.MSB != 0x00))
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF5(WORD pc)																			//JPS
{
	w.dcycles += 4; pcp(1); 
	if (!((Chipset.OVF != 0x00) ^ (Chipset.MSB != 0x00)))
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF6(WORD pc)																			//JNZ
{
	w.dcycles += 4; pcp(1); 
	if (! Chipset.Z)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF7(WORD pc)																			//JZR
{
	w.dcycles += 4; pcp(1); 
	if (Chipset.Z)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF8(WORD pc)																			//JEN
{
	w.dcycles += 4; pcp(1); 
	if (Chipset.E != 0x00)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oF9(WORD pc)																			//JEZ
{
	w.dcycles += 4; pcp(1); 
	if (Chipset.E == 0x00)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oFA(WORD pc)																			//JNC
{
	w.dcycles += 4; pcp(1); 
	if (! Chipset.CY)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oFB(WORD pc)																			//JCY
{
	w.dcycles += 4; pcp(1); 
	if (Chipset.CY)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oFC(WORD pc)																			//JLZ
{
	w.dcycles += 4; pcp(1); 
	if (Chipset.LDZ)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oFD(WORD pc)																			//JLN
{
	w.dcycles += 4; pcp(1); 
	if (! Chipset.LDZ)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oFE(WORD pc)																			//JRZ
{
	w.dcycles += 4; pcp(1); 
	if (Chipset.RDZ)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}
VOID __fastcall oFF(WORD pc)																			//JRN
{
	w.dcycles += 4; pcp(1); 
	if (! Chipset.RDZ)
	{
		BYTE d = Nread_nio1(pc);

		w.dcycles += 1; 	
		if (d > 0x7F)
			pcp(d - 0x100); 
		else
			pcp(d);
	}
}

