/*
 *   Ops.h
 *
 *   Copyright 2004 Olivier De Smet
 */

//
//   execute arithmetic and move opcodes, be carefull with all the flags ....
//
//   multiples definitions of the same operation for 1 byte or multibytes arguments
//

static __forceinline void NFRcopy1(BYTE a, BYTE b) // copy over Regs (with wrap)
{
	Chipset.R[a] = b;
	Chipset.LSB = ((b & 0x01) == 0x01);
	Chipset.RDZ = ((b & 0x0F) == 0x00);
	Chipset.Z = (b == 0x00);
	Chipset.MSB = ((b & 0x80) == 0x80);
	Chipset.LDZ = ((b & 0xF0) == 0x00);	
	Chipset.CY=FALSE;
	Chipset.OVF=FALSE;
}

static __forceinline void NFRcopy(BYTE *a, BYTE b, WORD s) // copy over Regs (with wrap)
{
	Chipset.Z = TRUE;
	Chipset.LSB = ((Chipset.R[b] & 0x01) == 0x01);
	Chipset.RDZ = ((Chipset.R[b] & 0x0F) == 0x00);
	do
	{
		*a = Chipset.R[b++];
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
		b &= 0x3F;
	} while (--s);
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);
	
	Chipset.CY=FALSE;
	Chipset.OVF=FALSE;
}

static __forceinline void NFcopy1(BYTE *a, BYTE b)	// copy over address
{
	Chipset.LSB = ((b & 0x01) == 0x01);
	Chipset.RDZ = ((b & 0x0F) == 0x00);
	Chipset.Z = (b == 0x00);
	*a = b;
	Chipset.MSB = ((b & 0x80) == 0x80);
	Chipset.LDZ = ((b & 0xF0) == 0x00);
	Chipset.CY=FALSE;
	Chipset.OVF=FALSE;
}

static __forceinline void NFcopy(BYTE *a, BYTE *b, WORD s)	// copy over address
{
	Chipset.Z = TRUE;

	Chipset.LSB = ((*b & 0x01) == 0x01);
	Chipset.RDZ = ((*b & 0x0F) == 0x00);
	do
	{
		if (*b != 0x00)
			Chipset.Z = FALSE;
		*a++ = *b++;
	} while (--s);
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY=FALSE;
	Chipset.OVF=FALSE;
}

static __forceinline void NFRstore1(BYTE a, BYTE b) // store over Regs (with wrap)
{
	Chipset.LSB = ((b & 0x01) == 0x01);
	Chipset.RDZ = ((b & 0x0F) == 0x00);
	Chipset.Z = (b == 0x00);
	Chipset.R[a] = b;
	Chipset.MSB = ((b & 0x80) == 0x80);
	Chipset.LDZ = ((b & 0xF0) == 0x00);
	Chipset.CY=FALSE;
	Chipset.OVF=FALSE;
}

static __forceinline void NFRstore(BYTE a, BYTE *b, WORD s) // store over Regs (with wrap)
{
	Chipset.Z = TRUE;

	Chipset.LSB = ((*b & 0x01) == 0x01);
	Chipset.RDZ = ((*b & 0x0F) == 0x00);
	do
	{
		if (*b != 0x00)
			Chipset.Z = FALSE;
		Chipset.R[a++] = *b++;
		a &= 0x3F;
	} while (--s);
	b--;
	Chipset.MSB = ((*b & 0x80) == 0x80);
	Chipset.LDZ = ((*b & 0xF0) == 0x00);

	Chipset.CY=FALSE;
	Chipset.OVF=FALSE;
}

static __forceinline BYTE NFread1(WORD d)
{
	BYTE a = Nread1(d);

	Chipset.LSB = ((a & 0x01) == 0x01);
	Chipset.RDZ = ((a & 0x0F) == 0x00);	
	Chipset.Z = (a == 0x00);
	Chipset.MSB = ((a & 0x80) == 0x80);
	Chipset.LDZ = ((a & 0xF0) == 0x00);
	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
	return a;
}

static __forceinline void NFread(BYTE *a, WORD d, WORD s)
{
	Nread(a, d, s);

	Chipset.Z = TRUE;

	Chipset.LSB = ((*a & 0x01) == 0x01);
	Chipset.RDZ = ((*a & 0x0F) == 0x00);	
	do
	{
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
	} while (--s);
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void NFwrite1(BYTE a, WORD d)
{
	Nwrite1(a, d);
	Chipset.LSB = ((a & 0x01) == 0x01);
	Chipset.RDZ = ((a & 0x0F) == 0x00);
	Chipset.Z = (a == 0x00);
	Chipset.MSB = ((a & 0x80) == 0x80);
	Chipset.LDZ = ((a & 0xF0) == 0x00);
	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void NFwrite(BYTE *a, WORD d, WORD s)
{
	Nwrite(a, d, (WORD)(s));
	Chipset.Z=TRUE;

	Chipset.LSB = ((*a & 0x01) == 0x01);
	Chipset.RDZ = ((*a & 0x0F) == 0x00);
	do
	{
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
	} while (--s);
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void Ninc1(BYTE *a)
{
	BYTE c = 1;

	if (Chipset.DCM)
	{
		BYTE d = *a & 0x0F; 
		
		if (d > 0x09) d -= 8;
		*a &= 0xF0;
		if ((d != 0x09) || (c == 0))
		{
			*a |= d + c;
			c = 0;
		}
		d = *a >> 4; if (d > 0x09) d -= 8;
		*a &= 0x0F;
		if ((d != 0x09) || (c == 0))
		{
			*a |= (d + c) << 4;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		if ((*a == 0xFF) && (c == 1))
		{
			Chipset.OVF = FALSE;
			*a = 0;
		}
		else
		{
			Chipset.OVF = (*a == 0x7F) && (c == 1);
			*a += c;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
	}
}

static __forceinline void Ninc(BYTE *a, WORD s)
{
	WORD i;
	BYTE c = 1;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			d = *a & 0x0F; if (d > 0x09) d -= 8;
			*a &= 0xF0;
			if ((d != 0x09) || (c == 0))
			{
				*a |= d + c;
				c = 0;
			}
			d = *a >> 4; if (d > 0x09) d -= 8;
			*a &= 0x0F;
			if ((d != 0x09) || (c == 0))
			{
				*a |= (d + c) << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		Chipset.OVF = FALSE;
		for (i = 0; i < s; i++)
		{
			if ((*a == 0xFF) && (c == 1))
				*a = 0;
			else
			{
				if (i == (s-1)) Chipset.OVF = (*a == 0x7F) && (c == 1);
				*a += c;
				c = 0;
			}
			if (i==0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
	}
}

static __forceinline void Ndec1(BYTE *a)
{
	BYTE c = 1;

	if (Chipset.DCM)
	{
		BYTE d = *a & 0x0F;

		*a &= 0xF0;
		if ((d == 0x00) && (c == 1))
		{
			*a |= 0x09;
		}
		else 
		{
			*a |= d - c;
			c = 0;
		}
		d = *a >> 4;
		*a &= 0x0F;
		if ((d == 0x00) && (c == 1))
		{
			*a |= 0x90;
		}
		else 
		{
			*a |= (d - c) << 4;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 0);
		Chipset.OVF = FALSE;
	}
	else
	{
		if ((*a == 0x00) && (c == 1))
		{
			Chipset.OVF = FALSE;
			*a = 0xFF;
		}
		else
		{
			Chipset.OVF = (*a == 0x80) && (c == 1);
			*a -= c;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 0);
	}
}

static __forceinline void Ndec(BYTE *a, WORD s)
{
	WORD i;
	BYTE c = 1;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			d = *a & 0x0F;
			*a &= 0xF0;
			if ((d == 0x00) && (c == 1))
			{
				*a |= 0x09;
			}
			else 
			{
				*a |= d - c;
				c = 0;
			}
			d = *a >> 4;
			*a &= 0x0F;
			if ((d == 0x00) && (c == 1))
			{
				*a |= 0x90;
			}
			else 
			{
				*a |= (d - c) << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 0);
		Chipset.OVF = FALSE;
	}
	else
	{
		Chipset.OVF = FALSE;
		for (i=0; i < s; i++)
		{
			if ((*a == 0x00) && (c == 1))
				*a = 0xFF;
			else
			{
				if (i == (s-1)) Chipset.OVF = (*a == 0x80) && (c == 1);
				*a -= c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 0);
	}
}

static __forceinline void NRadd1(BYTE *a, BYTE b)		// add over registers with wrap
{
	BYTE c = 0;
	BYTE bb;

	if (Chipset.DCM)
	{
		BYTE d;

		bb = Chipset.R[b] & 0x0F;
		d = *a & 0x0F; if (d > 0x09) d -= 8;
		*a &= 0xF0;
		if ((d + bb + c) > 0x09)
		{
			*a |= d + bb + c - 10;
			c = 1;
		}
		else 
		{
			*a |= d + bb + c;
			c = 0;
		}
		bb = Chipset.R[b] >> 4;
		d = *a >> 4; if (d > 0x09) d -= 8;
		*a &= 0x0F;
		if ((d + bb + c) > 0x09)
		{
			*a |= (d + bb + c - 10) << 4;
			c = 1;
		}
		else 
		{
			*a |= (d + bb + c) << 4;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		bb = Chipset.R[b];
		Chipset.OVF = ((*a & 0x80) == 0x80);
		if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
		{
			*a += bb + c;
			c = 1;
		}
		else
		{
			*a += bb + c;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void NRadd(BYTE *a, BYTE b, WORD s)		// add over registers with wrap
{
	WORD i;
	BYTE c = 0;
	BYTE bb;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			bb = Chipset.R[b] & 0x0F;
			d = *a & 0x0F; if (d > 0x09) d -= 8;
			*a &= 0xF0;
			if ((d + bb + c) > 0x09)
			{
				*a |= d + bb + c - 10;
				c = 1;
			}
			else 
			{
				*a |= d + bb + c;
				c = 0;
			}
			bb = Chipset.R[b] >> 4;
			d = *a >> 4; if (d > 0x09) d -= 8;
			*a &= 0x0F;
			if ((d + bb + c) > 0x09)
			{
				*a |= (d + bb + c - 10) << 4;
				c = 1;
			}
			else 
			{
				*a |= (d + bb + c) << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
			b &= 0x3F;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		for (i = 0; i < s; i++)
		{
			bb = Chipset.R[b];
			if (i == (s-1)) Chipset.OVF = ((*a & 0x80) == 0x80);
			if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
			{
				*a += bb + c;
				c = 1;
			}
			else
			{
				*a += bb + c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
			b &= 0x3F;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void Nadd1(BYTE *a, BYTE b)		// add over memory
{
	BYTE c = 0;
	BYTE bb;

	if (Chipset.DCM)
	{
		BYTE d;

		bb = b & 0x0F;
		d = *a & 0x0F; if (d > 0x09) d -= 8;
		*a &= 0xF0;
		if ((d + bb + c) > 0x09)
		{
			*a |= d + bb + c - 10;
			c = 1;
		}
		else 
		{
			*a |= d + bb + c;
			c = 0;
		}
		bb = b >> 4;
		d = *a >> 4; if (d > 0x09) d -= 8;
		*a &= 0x0F;
		if ((d + bb + c) > 0x09)
		{
			*a |= (d + bb + c - 10) << 4;
			c = 1;
		}
		else 
		{
			*a |= (d + bb + c) << 4;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		bb = b;
		Chipset.OVF = ((*a & 0x80) == 0x80);
		if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
		{
			*a += bb + c;
			c = 1;
		}
		else
		{
			*a += bb + c;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void Nadd(BYTE *a, BYTE *b, WORD s)		// add over memory
{
	WORD i;
	BYTE c = 0;
	BYTE bb;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			bb = *b & 0x0F;
			d = *a & 0x0F; if (d > 0x09) d -= 8;
			*a &= 0xF0;
			if ((d + bb + c) > 0x09)
			{
				*a |= d + bb + c - 10;
				c = 1;
			}
			else 
			{
				*a |= d + bb + c;
				c = 0;
			}
			bb = *b >> 4;
			d = *a >> 4; if (d > 0x09) d -= 8;
			*a &= 0x0F;
			if ((d + bb + c) > 0x09)
			{
				*a |= (d + bb + c - 10) << 4;
				c = 1;
			}
			else 
			{
				*a |= (d + bb + c) << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		for (i = 0; i < s; i++)
		{
			bb = *b;
			if (i == (s-1)) Chipset.OVF = ((*a & 0x80) == 0x80);
			if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
			{
				*a += bb + c;
				c = 1;
			}
			else
			{
				*a += bb + c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void NRsub1(BYTE *a, BYTE b)		// sub over registers with wrap
{
	BYTE c = 1;
	BYTE bb;

	if (Chipset.DCM)
	{
		BYTE d;

		bb = Chipset.R[b] & 0x0F;
		d = *a & 0x0F;
		*a &= 0xF0;
		if (d + c + 15 - bb > 15)
		{
			*a |= d + c + 9 - bb - 10;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			*a |= d;
			c = 0;
		}
		bb = Chipset.R[b] >> 4;
		d = *a >> 4;
		*a &= 0x0F;
		if (d + c + 15 - bb > 15)
		{
			*a |= (d + c + 9 - bb - 10) << 4;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			*a |= d << 4;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		bb = ~(Chipset.R[b]);
		Chipset.OVF = ((*a & 0x80) == 0x80);
		if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
		{
			*a += bb + c;
			c = 1;
		}
		else
		{
			*a += bb + c;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void NRsub(BYTE *a, BYTE b, WORD s)		// sub over registers with wrap
{
	WORD i;
	BYTE c = 1;
	BYTE bb;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			bb = Chipset.R[b] & 0x0F;
			d = *a & 0x0F;
			*a &= 0xF0;
			if (d + c + 15 - bb > 15)
			{
				*a |= d + c + 9 - bb - 10;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				*a |= d;
				c = 0;
			}
			bb = Chipset.R[b] >> 4;
			d = *a >> 4;
			*a &= 0x0F;
			if (d + c + 15 - bb > 15)
			{
				*a |= (d + c + 9 - bb - 10) << 4;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				*a |= d << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
			b &= 0x3F;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		for (i = 0; i < s; i++)
		{
			bb = ~(Chipset.R[b]);
			if (i == (s-1)) Chipset.OVF = ((*a & 0x80) == 0x80);
			if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
			{
				*a += bb + c;
				c = 1;
			}
			else
			{
				*a += bb + c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
			b &= 0x3F;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void Nsub1(BYTE *a, BYTE b)		// sub over memory
{
	BYTE c = 1;
	BYTE bb;

	if (Chipset.DCM)
	{
		BYTE d;

		bb = b & 0x0F;
		d = *a & 0x0F;
		*a &= 0xF0;
		if (d + c + 15 - bb > 15)
		{
			*a |= d + c + 9 - bb - 10;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			*a |= d;
			c = 0;
		}
		bb = b >> 4;
		d = *a >> 4;
		*a &= 0x0F;
		if (d + c + 15 - bb > 15)
		{
			*a |= (d + c + 9 - bb - 10) << 4;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			*a |= d << 4;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		bb = ~(b);
		Chipset.OVF = ((*a & 0x80) == 0x80);
		if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
		{
			*a += bb + c;
			c = 1;
		}
		else
		{
			*a += bb + c;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void Nsub(BYTE *a, BYTE *b, WORD s)		// sub over memory
{
	WORD i;
	BYTE c = 1;
	BYTE bb;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			bb = *b & 0x0F;
			d = *a & 0x0F;
			*a &= 0xF0;
			if (d + c + 15 - bb > 15)
			{
				*a |= d + c + 9 - bb - 10;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				*a |= d;
				c = 0;
			}
			bb = *b >> 4;
			d = *a >> 4;
			*a &= 0x0F;
			if (d + c + 15 - bb > 15)
			{
				*a |= (d + c + 9 - bb - 10) << 4;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				*a |= d << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		for (i = 0; i < s; i++)
		{
			bb = ~(*b);
			if (i == (s-1)) Chipset.OVF = ((*a & 0x80) == 0x80);
			if ((WORD)(*a) + (WORD)(bb) + (WORD)(c) > 0xFF)
			{
				*a += bb + c;
				c = 1;
			}
			else
			{
				*a += bb + c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void NRcm1(BYTE *a, BYTE b)		// cm over registers with wrap
{
	BYTE c = 1;
	BYTE bb;
	BYTE r;

	if (Chipset.DCM)
	{
		BYTE d;

		r = *a;
		bb = Chipset.R[b] & 0x0F;
		d = r & 0x0F;
		r &= 0xF0;
		if (d + c + 15 - bb > 15)
		{
			r |= d + c + 9 - bb - 10;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			r |= d;
			c = 0;
		}
		bb = Chipset.R[b] >> 4;
		d = r >> 4;
		r &= 0x0F;
		if (d + c + 15 - bb > 15)
		{
			r |= (d + c + 9 - bb - 10) << 4;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			r |= d << 4;
			c = 0;
		}
		Chipset.LSB = ((r & 0x01) == 0x01);
		Chipset.RDZ = ((r & 0x0F) == 0x00);
		Chipset.Z = (r == 0x00);
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		r = *a;
		Chipset.OVF = ((r & 0x80) == 0x80);
		bb = ~(Chipset.R[b]);
		if ((WORD)(r) + (WORD)(bb) + (WORD)(c) > 0xFF)
		{
			r += bb + c;
			c = 1;
		}
		else
		{
			r += bb + c;
			c = 0;
		}
		Chipset.LSB = ((r & 0x01) == 0x01);
		Chipset.RDZ = ((r & 0x0F) == 0x00);
		Chipset.Z = (r == 0x00);
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void NRcm(BYTE *a, BYTE b, WORD s)		// cm over registers with wrap
{
	WORD i;
	BYTE c = 1;
	BYTE bb;
	BYTE r;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			r = *a;

			bb = Chipset.R[b] & 0x0F;
			d = r & 0x0F;
			r &= 0xF0;
			if (d + c + 15 - bb > 15)
			{
				r |= d + c + 9 - bb - 10;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				r |= d;
				c = 0;
			}
			bb = Chipset.R[b] >> 4;
			d = r >> 4;
			r &= 0x0F;
			if (d + c + 15 - bb > 15)
			{
				r |= (d + c + 9 - bb - 10) << 4;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				r |= d << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((r & 0x01) == 0x01);
				Chipset.RDZ = ((r & 0x0F) == 0x00);
			}
			if (r != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
			b &= 0x3F;
		}
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		for (i = 0; i < s; i++)
		{
			r = *a;

			if (i == (s-1)) Chipset.OVF = ((r & 0x80) == 0x80);
			bb = ~(Chipset.R[b]);
			if ((WORD)(r) + (WORD)(bb) + (WORD)(c) > 0xFF)
			{
				r += bb + c;
				c = 1;
			}
			else
			{
				r += bb + c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((r & 0x01) == 0x01);
				Chipset.RDZ = ((r & 0x0F) == 0x00);
			}
			if (r != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
			b &= 0x3F;
		}
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void Ncm1(BYTE *a, BYTE b)		// cm over memory
{
	BYTE c = 1;
	BYTE bb;
	BYTE r;

	if (Chipset.DCM)
	{
		BYTE d;

		r = *a;
		bb = b & 0x0F;
		d = r & 0x0F;
		r &= 0xF0;
		if (d + c + 15 - bb > 15)
		{
			r |= d + c + 9 - bb - 10;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			r |= d;
			c = 0;
		}
		bb = b >> 4;
		d = r >> 4;
		r &= 0x0F;
		if (d + c + 15 - bb > 15)
		{
			r |= (d + c + 9 - bb - 10) << 4;
			c = 1;
		}
		else
		{
			d = (d + c + 9 - bb) & 0x0F; 
			if (d > 0x09) d -= 8;
			r |= d << 4;
			c = 0;
		}
		Chipset.LSB = ((r & 0x01) == 0x01);
		Chipset.RDZ = ((r & 0x0F) == 0x00);
		Chipset.Z = (r == 0x00);
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		r = *a;
		Chipset.OVF = ((r & 0x80) == 0x80);
		bb = ~(b);
		if ((WORD)(r) + (WORD)(bb) + (WORD)(c) > 0xFF)
		{
			r += bb + c;
			c = 1;
		}
		else
		{
			r += bb + c;
			c = 0;
		}
		Chipset.LSB = ((r & 0x01) == 0x01);
		Chipset.RDZ = ((r & 0x0F) == 0x00);
		Chipset.Z = (r == 0x00);
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void Ncm(BYTE *a, BYTE *b, WORD s)		// cm over memory
{
	WORD i;
	BYTE c = 1;
	BYTE bb;
	BYTE r;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s; i++)
		{
			r = *a;

			bb = *b & 0x0F;
			d = r & 0x0F;
			r &= 0xF0;
			if (d + c + 15 - bb > 15)
			{
				r |= d + c + 9 - bb - 10;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				r |= d;
				c = 0;
			}
			bb = *b >> 4;
			d = r >> 4;
			r &= 0x0F;
			if (d + c + 15 - bb > 15)
			{
				r |= (d + c + 9 - bb - 10) << 4;
				c = 1;
			}
			else
			{
				d = (d + c + 9 - bb) & 0x0F; 
				if (d > 0x09) d -= 8;
				r |= d << 4;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((r & 0x01) == 0x01);
				Chipset.RDZ = ((r & 0x0F) == 0x00);
			}
			if (r != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
		}
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
		Chipset.OVF = FALSE;
	}
	else
	{
		for (i = 0; i < s; i++)
		{
			r = *a;

			if (i == (s-1)) Chipset.OVF = ((r & 0x80) == 0x80);
			bb = ~(*b);
			if ((WORD)(r) + (WORD)(bb) + (WORD)(c) > 0xFF)
			{
				r += bb + c;
				c = 1;
			}
			else
			{
				r += bb + c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((r & 0x01) == 0x01);
				Chipset.RDZ = ((r & 0x0F) == 0x00);
			}
			if (r != 0x00)
				Chipset.Z = FALSE;
			a++;
			b++;
		}
		Chipset.MSB = ((r & 0x80) == 0x80);
		Chipset.LDZ = ((r & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
		Chipset.OVF ^= ((bb & 0x80) == 0x80) ^ Chipset.MSB ^ Chipset.CY;
	}
}

static __forceinline void NRand(BYTE *a, BYTE b, WORD s)		// and over registers with wrap
{
	WORD i;

	Chipset.Z = TRUE;
	for (i = 0; i < s ; i++)
	{
		*a &= Chipset.R[b];
		if (i == 0)
		{
			Chipset.LSB = ((*a & 0x01) == 0x01);
			Chipset.RDZ = ((*a & 0x0F) == 0x00);
		}
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
		b++;
		b &= 0x3F;
	}
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void Nand(BYTE *a, BYTE *b, WORD s)		// and over memory
{
	WORD i;

	Chipset.Z = TRUE;
	for (i = 0; i < s ; i++)
	{
		*a &= *b;
		if (i == 0)
		{
			Chipset.LSB = ((*a & 0x01) == 0x01);
			Chipset.RDZ = ((*a & 0x0F) == 0x00);
		}
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
		b++;
	}
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void NRor1(BYTE *a, BYTE b)		// or over registers with wrap
{
	*a |= Chipset.R[b];
	Chipset.LSB = ((*a & 0x01) == 0x01);
	Chipset.RDZ = ((*a & 0x0F) == 0x00);
	Chipset.Z = (*a == 0x00);
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);
	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void NRor(BYTE *a, BYTE b, WORD s)		// or over registers with wrap
{
	WORD i;

	Chipset.Z = TRUE;
	for (i = 0; i < s ; i++)
	{
		*a |= Chipset.R[b];
		if (i == 0)
		{
			Chipset.LSB = ((*a & 0x01) == 0x01);
			Chipset.RDZ = ((*a & 0x0F) == 0x00);
		}
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
		b++;
		b &= 0x3F;
	}
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void NRxr1(BYTE *a, BYTE b)		// xor over registers with wrap
{
	*a ^= Chipset.R[b];
	Chipset.LSB = ((*a & 0x01) == 0x01);
	Chipset.RDZ = ((*a & 0x0F) == 0x00);
	Chipset.Z = (*a == 0x00);
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);
	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void NRxr(BYTE *a, BYTE b, WORD s)		// xor over registers with wrap
{
	WORD i;

	Chipset.Z = TRUE;
	for (i = 0; i < s ; i++)
	{
		*a ^= Chipset.R[b];
		if (i == 0)
		{
			Chipset.LSB = ((*a & 0x01) == 0x01);
			Chipset.RDZ = ((*a & 0x0F) == 0x00);
		}
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
		b++;
		b &= 0x3F;
	}
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void Nnot1(BYTE *a)		// one or nine complement
{ 
	if (Chipset.DCM)
	{
		BYTE d = *a & 0x0F;
		
		if (d > 0x09) d -= 8;
		*a = (*a & 0xF0) | (9 - d);
		d = *a >> 4; if (d > 0x09) d -= 8;
		*a = (*a & 0x0F) | ((9 - d) << 4);
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = FALSE;
		Chipset.OVF = FALSE;
	}
	else
	{
		*a = ~(*a);
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = FALSE;
		Chipset.OVF = FALSE;
	}
}

static __forceinline void Nnot(BYTE *a, WORD s)		// one or nine complement
{ 
	WORD i;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s ; i++)
		{
			d = *a & 0x0F; if (d > 0x09) d -= 8;
			*a = (*a & 0xF0) | (9 - d);
			d = *a >> 4; if (d > 0x09) d -= 8;
			*a = (*a & 0x0F) | ((9 - d) << 4);
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = FALSE;
		Chipset.OVF = FALSE;
	}
	else
	{
		for (i = 0; i < s ; i++)
		{
			*a = ~(*a);
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = FALSE;
		Chipset.OVF = FALSE;
	}
}

static __forceinline void Nneg1(BYTE *a)		// two or ten complement
{
	BYTE c = 1;

	Chipset.OVF = FALSE;
	if (Chipset.DCM)
	{
		BYTE d = *a & 0x0F;

		*a &= 0xF0;
		if (c > d)
		{
			*a |= c + 9 - d - 10;
			c = 1;
		}
		else
		{
			d = (c + 9 - d) & 0x0F; 
			if (d > 0x09) d -= 8;
			*a |= d;
			c = 0;
		}
		d = *a >> 4;
		*a &= 0x0F;
		if (c > d)
		{
			*a |= (c + 9 - d - 10) << 4;
			c = 1;
		}
		else
		{
			d = (c + 9 - d) & 0x0F; 
			if (d > 0x09) d -= 8;
			*a |= d << 4;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
	}
	else
	{
		if ((*a == 0x80) && (c == 1))
			Chipset.OVF = TRUE;
		*a = ~(*a);
		if ((*a == 0xFF) && (c == 1))
			*a = 0;
		else
		{
			*a += c;
			c = 0;
		}
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
	}
}

static __forceinline void Nneg(BYTE *a, WORD s)		// two or ten complement
{
	WORD i;
	BYTE c = 1;

	Chipset.OVF = FALSE;
	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		BYTE d;

		for (i = 0; i < s ; i++)
		{
			d = *a & 0x0F;
			*a &= 0xF0;
			if (c > d)
			{
				*a |= c + 9 - d - 10;
				c = 1;
			}
			else
			{
				d = (c + 9 - d) & 0x0F; 
				if (d > 0x09) d -= 8;
				*a |= d;
				c = 0;
			}
			d = *a >> 4;
			*a &= 0x0F;
			if (c > d)
			{
				*a |= (c + 9 - d - 10) << 4;
				c = 1;
			}
			else
			{
				d = (c + 9 - d) & 0x0F; 
				if (d > 0x09) d -= 8;
				*a |= d << 4;
				c = 0;
			}

			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
	}
	else
	{
		
		for (i = 0; i < s ; i++)
		{
			if ((i == (s-1)) && (*a == 0x80) && (c == 1))
				Chipset.OVF = TRUE;
			*a = ~(*a);
			if ((*a == 0xFF) && (c == 1))
				*a = 0;
			else
			{
				*a += c;
				c = 0;
			}
			if (i == 0)
			{
				Chipset.LSB = ((*a & 0x01) == 0x01);
				Chipset.RDZ = ((*a & 0x0F) == 0x00);
			}
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
	}
}

static __forceinline void Nsl1(BYTE *a)		// left shift
{
	BYTE c = 0;
	BYTE d;

	if (Chipset.DCM)
	{
		Chipset.RDZ = TRUE;
		Chipset.LSB = FALSE;
		d = *a >> 4;
		*a = ((*a & 0x0F) << 4) | c;
		c = d;
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.E = c;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		Chipset.RDZ = ((*a & 0x07) == 0x00);
		Chipset.LSB = FALSE;
		Chipset.OVF = ((*a & 0x80) == 0x80);
		d = (*a & 0x80) >> 7;
		*a = ((*a & 0x7F) << 1) | c;
		c = d;
		Chipset.Z = (*a == 0x00);
		Chipset.OVF ^= ((*a & 0x80) == 0x80);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
	}
}

static __forceinline void Nsl(BYTE *a, WORD s)		// left shift
{
	BYTE c = 0;
	BYTE d;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		Chipset.RDZ = TRUE;
		Chipset.LSB = FALSE;
		do
		{
			d = *a >> 4;
			*a = ((*a & 0x0F) << 4) | c;
			c = d;
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		} while (--s);
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.E = c;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		WORD i;

		Chipset.RDZ = ((*a & 0x07) == 0x00);
		Chipset.LSB = FALSE;
		for (i = 0; i < s ; i++)
		{
			if (i == (s-1)) Chipset.OVF = ((*a & 0x80) == 0x80);
			d = (*a & 0x80) >> 7;
			*a = ((*a & 0x7F) << 1) | c;
			c = d;
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.OVF ^= ((*a & 0x80) == 0x80);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
	}
}

static __forceinline void Nesl1(BYTE *a)		// extended left shift
{
	BYTE c;
	BYTE d;

	if (Chipset.DCM)
	{
		c = Chipset.E;
		Chipset.RDZ = (c == 0x00);
		Chipset.LSB = ((c & 0x01) == 0x01);
		d = *a >> 4;
		*a = ((*a & 0x0F) << 4) | c;
		c = d;
		Chipset.Z = (*a == 0x00);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.E = c;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		c = Chipset.CY;
		Chipset.RDZ = ((*a & 0x07) == 0x00) && (c == 0);
		Chipset.LSB = (c == 1);
		Chipset.OVF = ((*a & 0x80) == 0x80);
		d = (*a & 0x80) >> 7;
		*a = ((*a & 0x7F) << 1) | c;
		c = d;
		Chipset.Z = (*a == 0x00);
		Chipset.OVF ^= ((*a & 0x80) == 0x80);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);
		Chipset.CY = (c == 1);
	}
}

static __forceinline void Nesl(BYTE *a, WORD s)		// extended left shift
{
	BYTE c;
	BYTE d;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		c = Chipset.E;
		Chipset.RDZ = (c == 0x00);
		Chipset.LSB = ((c & 0x01) == 0x01);
		do
		{
			d = *a >> 4;
			*a = ((*a & 0x0F) << 4) | c;
			c = d;
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		} while (--s);
		a--;
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.E = c;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		WORD i;

		c = Chipset.CY;
		Chipset.RDZ = ((*a & 0x07) == 0x00) && (c == 0);
		Chipset.LSB = (c == 1);
		for (i = 0; i < s ; i++)
		{
			if (i == (s-1)) Chipset.OVF = ((*a & 0x80) == 0x80);
			d = (*a & 0x80) >> 7;
			*a = ((*a & 0x7F) << 1) | c;
			c = d;
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a++;
		}
		a--;
		Chipset.OVF ^= ((*a & 0x80) == 0x80);
		Chipset.MSB = ((*a & 0x80) == 0x80);
		Chipset.LDZ = ((*a & 0xF0) == 0x00);

		Chipset.CY = (c == 1);
	}
}

static __forceinline void Nsr1(BYTE *a)		// right shift
{
	BYTE c = 0;
	BYTE d;

	if (Chipset.DCM)
	{
		Chipset.LDZ = TRUE;
		Chipset.MSB = FALSE;
		d = (*a & 0x0F) << 4;
		*a = (*a >> 4) | c;
		c = d;
		Chipset.Z = (*a == 0x00);
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.E = c >> 4;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		Chipset.LDZ = ((*a & 0xE0) == 0x00);
		Chipset.MSB = FALSE;
		d = (*a & 0x01) << 7;
		*a = (*a >> 1) | c;
		c = d; 
		Chipset.Z = (*a == 0x00);
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.CY = (c == 0x80);
		Chipset.OVF = FALSE;
	}
}

static __forceinline void Nsr(BYTE *a, WORD s)		// right shift
{
	BYTE c = 0;
	BYTE d;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		Chipset.LDZ = TRUE;
		Chipset.MSB = FALSE;
		do
		{
			d = (*a & 0x0F) << 4;
			*a = (*a >> 4) | c;
			c = d;
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a--;
		} while (--s);
		a++;
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);

		Chipset.E = c >> 4;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		Chipset.LDZ = ((*a & 0xE0) == 0x00);
		Chipset.MSB = FALSE;
		do
		{
			d = (*a & 0x01) << 7;
			*a = (*a >> 1) | c;
			c = d; 
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a--;
		} while (--s);
		a++;
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);

		Chipset.CY = (c == 0x80);
		Chipset.OVF = FALSE;
	}
}

static __forceinline void Nesr1(BYTE *a)		// extended right shift
{
	BYTE c;
	BYTE d;

	if (Chipset.DCM)
	{
		c = Chipset.E << 4;
		Chipset.LDZ = (c == 0x00);
		Chipset.MSB = (c == 0x80);
		d = (*a & 0x0F) << 4;
		*a = (*a >> 4) | c;
		c = d;
		Chipset.Z = (*a == 0x00);
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.E = c >> 4;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		c = Chipset.CY << 7;
		Chipset.MSB = Chipset.CY;
		Chipset.LDZ = ((*a & 0xE0) == 0x00) && (c == 0);
		d = (*a & 0x01) << 7;
		*a = (*a >> 1) | c;
		c = d; 
		Chipset.Z = (*a == 0x00);
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);
		Chipset.CY = (c == 0x80);
		Chipset.OVF = FALSE;
	}
}

static __forceinline void Nesr(BYTE *a, WORD s)		// extended right shift
{
	BYTE c;
	BYTE d;

	Chipset.Z = TRUE;
	if (Chipset.DCM)
	{
		c = Chipset.E << 4;
		Chipset.LDZ = (c == 0x00);
		Chipset.MSB = (c == 0x80);
		do
		{
			d = (*a & 0x0F) << 4;
			*a = (*a >> 4) | c;
			c = d;
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a--;
		} while (--s);
		a++;
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);

		Chipset.E = c >> 4;
		Chipset.OVF = FALSE;
		Chipset.CY = FALSE;
	}
	else
	{
		c = Chipset.CY << 7;
		Chipset.MSB = Chipset.CY;
		Chipset.LDZ = ((*a & 0xE0) == 0x00) && (c == 0);
		do
		{
			d = (*a & 0x01) << 7;
			*a = (*a >> 1) | c;
			c = d; 
			if (*a != 0x00)
				Chipset.Z = FALSE;
			a--;
		} while (--s);
		a++;
		Chipset.LSB = ((*a & 0x01) == 0x01);
		Chipset.RDZ = ((*a & 0x0F) == 0x00);

		Chipset.CY = (c == 0x80);
		Chipset.OVF = FALSE;
	}
}

static __forceinline void Ntst1(BYTE a)			// test value
{
	Chipset.LSB = ((a & 0x01) == 0x01);
	Chipset.RDZ = ((a & 0x0F) == 0x00);
	Chipset.Z = (a == 0x00);
	Chipset.MSB = ((a & 0x80) == 0x80);
	Chipset.LDZ = ((a & 0xF0) == 0x00);
	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

static __forceinline void Ntst(BYTE *a, WORD s)
{
	Chipset.Z = TRUE;

	Chipset.LSB = ((*a & 0x01) == 0x01);
	Chipset.RDZ = ((*a & 0x0F) == 0x00);
	do
	{
		if (*a != 0x00)
			Chipset.Z = FALSE;
		a++;
	} while (--s);
	a--;
	Chipset.MSB = ((*a & 0x80) == 0x80);
	Chipset.LDZ = ((*a & 0xF0) == 0x00);

	Chipset.CY = FALSE;
	Chipset.OVF = FALSE;
}

